@@ -2,12 +2,14 @@ from __future__ import annotations
 
 import os
 import textwrap
+import typing as t
 
 import click
+import jsonschema
 
 from ..catalog import CUSTOM_SCHEMA_NAMES, SCHEMA_CATALOG
 from ..checker import SchemaChecker
-from ..formats import KNOWN_FORMATS, RegexFormatBehavior
+from ..formats import KNOWN_FORMATS, RegexVariantName
 from ..instance_loader import InstanceLoader
 from ..parsers import SUPPORTED_FILE_FORMATS
 from ..reporter import REPORTER_BY_NAME, Reporter
@@ -18,9 +20,8 @@ from ..schema_loader import (
     SchemaLoaderBase,
 )
 from ..transforms import TRANSFORM_LIBRARY
-from .param_types import CommaDelimitedList
+from .param_types import CommaDelimitedList, LazyBinaryReadFile, ValidatorClassName
 from .parse_result import ParseResult, SchemaLoadingMode
-from .warnings import deprecation_warning_callback
 
 BUILTIN_SCHEMA_NAMES = [f"vendor.{k}" for k in SCHEMA_CATALOG.keys()] + [
     f"custom.{k}" for k in CUSTOM_SCHEMA_NAMES
@@ -69,8 +70,7 @@ including the following formats by default:
 \b
 For the "regex" format, there are multiple modes which can be specified with
 '--format-regex':
-    default  |  best effort check
-    disabled |  do not check the regex format
+    default  |  check that the string is a valid ECMAScript regex
     python   |  check that the string is a valid python regex
 
 \b
@@ -91,7 +91,17 @@ The '--disable-formats' flag supports the following formats:
     help=(
         "The path to a file containing the JSON Schema to use or an "
         "HTTP(S) URI for the schema. If a remote file is used, "
-        "it will be downloaded and cached locally based on mtime."
+        "it will be downloaded and cached locally based on mtime. "
+        "Use '-' for stdin."
+    ),
+    metavar="[PATH|URI]",
+)
+@click.option(
+    "--base-uri",
+    help=(
+        "Override the base URI for the schema. The default behavior is to "
+        "follow the behavior specified by the JSON Schema spec, which is to "
+        "prefer an explicit '$id' and failover to the retrieval URI."
     ),
 )
 @click.option(
@@ -120,17 +130,6 @@ The '--disable-formats' flag supports the following formats:
         "Defaults to the last slash-delimited part of the URI."
     ),
 )
-@click.option(
-    "--disable-format",
-    is_flag=True,
-    help="{deprecated} Disable all format checks in the schema.",
-    callback=deprecation_warning_callback(
-        "--disable-format",
-        is_flag=True,
-        append_message="Users should now pass '--disable-formats \"*\"' for "
-        "the same functionality.",
-    ),
-)
 @click.option(
     "--disable-formats",
     multiple=True,
@@ -145,8 +144,8 @@ The '--disable-formats' flag supports the following formats:
         "Set the mode of format validation for regexes. "
         "If `--disable-formats regex` is used, this option has no effect."
     ),
-    default=RegexFormatBehavior.default.value,
-    type=click.Choice([x.value for x in RegexFormatBehavior], case_sensitive=False),
+    default=RegexVariantName.default.value,
+    type=click.Choice([x.value for x in RegexVariantName], case_sensitive=False),
 )
 @click.option(
     "--default-filetype",
@@ -174,13 +173,27 @@ The '--disable-formats' flag supports the following formats:
 )
 @click.option(
     "--fill-defaults",
-    help="Autofill 'default' values prior to validation.",
+    help=(
+        "Autofill 'default' values prior to validation. "
+        "This may conflict with certain third-party validators used with "
+        "'--validator-class'"
+    ),
     is_flag=True,
 )
+@click.option(
+    "--validator-class",
+    help=(
+        "The fully qualified name of a python validator to use in place of "
+        "the 'jsonschema' library validators, in the form of '<package>:<class>'. "
+        "The validator must be importable in the same environment where "
+        "'check-jsonschema' is run."
+    ),
+    type=ValidatorClassName(),
+)
 @click.option(
     "-o",
     "--output-format",
-    help="Which output format to use",
+    help="Which output format to use.",
     type=click.Choice(tuple(REPORTER_BY_NAME.keys()), case_sensitive=False),
     default="text",
 )
@@ -207,39 +220,46 @@ The '--disable-formats' flag supports the following formats:
     help="Reduce output verbosity",
     count=True,
 )
-@click.argument("instancefiles", required=True, nargs=-1)
+@click.argument(
+    "instancefiles", required=True, nargs=-1, type=LazyBinaryReadFile("rb", lazy=True)
+)
 def main(
     *,
     schemafile: str | None,
     builtin_schema: str | None,
+    base_uri: str | None,
     check_metaschema: bool,
     no_cache: bool,
     cache_filename: str | None,
-    disable_format: bool,
     disable_formats: tuple[list[str], ...],
     format_regex: str,
     default_filetype: str,
     traceback_mode: str,
     data_transform: str | None,
     fill_defaults: bool,
+    validator_class: type[jsonschema.protocols.Validator] | None,
     output_format: str,
     verbose: int,
     quiet: int,
-    instancefiles: tuple[str, ...],
+    instancefiles: tuple[t.BinaryIO, ...],
 ) -> None:
     args = ParseResult()
 
     args.set_schema(schemafile, builtin_schema, check_metaschema)
+    args.set_validator(validator_class)
+
+    args.base_uri = base_uri
     args.instancefiles = instancefiles
 
     normalized_disable_formats: tuple[str, ...] = tuple(
         f for sublist in disable_formats for f in sublist
     )
-    if disable_format or "*" in normalized_disable_formats:
+    if "*" in normalized_disable_formats:
         args.disable_all_formats = True
     else:
         args.disable_formats = normalized_disable_formats
-    args.format_regex = RegexFormatBehavior(format_regex)
+
+    args.format_regex = RegexVariantName(format_regex)
     args.disable_cache = no_cache
     args.default_filetype = default_filetype
     args.fill_defaults = fill_defaults
@@ -264,13 +284,19 @@ def main(
 
 def build_schema_loader(args: ParseResult) -> SchemaLoaderBase:
     if args.schema_mode == SchemaLoadingMode.metaschema:
-        return MetaSchemaLoader()
+        return MetaSchemaLoader(base_uri=args.base_uri)
     elif args.schema_mode == SchemaLoadingMode.builtin:
         assert args.schema_path is not None
-        return BuiltinSchemaLoader(args.schema_path)
+        return BuiltinSchemaLoader(args.schema_path, base_uri=args.base_uri)
     elif args.schema_mode == SchemaLoadingMode.filepath:
         assert args.schema_path is not None
-        return SchemaLoader(args.schema_path, args.cache_filename, args.disable_cache)
+        return SchemaLoader(
+            args.schema_path,
+            args.cache_filename,
+            args.disable_cache,
+            base_uri=args.base_uri,
+            validator_class=args.validator_class,
+        )
     else:
         raise NotImplementedError("no valid schema option provided")
 
