@@ -12,7 +12,7 @@ from ..formats import FormatOptions, make_format_checker
 from ..parsers import ParserSet
 from ..utils import is_url_ish
 from .errors import UnsupportedUrlScheme
-from .readers import HttpSchemaReader, LocalSchemaReader
+from .readers import HttpSchemaReader, LocalSchemaReader, StdinSchemaReader
 from .resolver import make_reference_registry
 
 
@@ -47,7 +47,7 @@ def _extend_with_default(
 class SchemaLoaderBase:
     def get_validator(
         self,
-        path: pathlib.Path,
+        path: pathlib.Path | str,
         instance_doc: dict[str, t.Any],
         format_opts: FormatOptions,
         fill_defaults: bool,
@@ -56,16 +56,22 @@ class SchemaLoaderBase:
 
 
 class SchemaLoader(SchemaLoaderBase):
+    validator_class: type[jsonschema.protocols.Validator] | None = None
+
     def __init__(
         self,
         schemafile: str,
         cache_filename: str | None = None,
         disable_cache: bool = False,
+        base_uri: str | None = None,
+        validator_class: type[jsonschema.protocols.Validator] | None = None,
     ) -> None:
         # record input parameters (these are not to be modified)
         self.schemafile = schemafile
         self.cache_filename = cache_filename
         self.disable_cache = disable_cache
+        self.base_uri = base_uri
+        self.validator_class = validator_class
 
         # if the schema location is a URL, which may include a file:// URL, parse it
         self.url_info = None
@@ -76,15 +82,22 @@ class SchemaLoader(SchemaLoaderBase):
         self._parsers = ParserSet()
 
         # setup a schema reader lazily, when needed
-        self._reader: LocalSchemaReader | HttpSchemaReader | None = None
+        self._reader: LocalSchemaReader | HttpSchemaReader | StdinSchemaReader | None = (
+            None
+        )
 
     @property
-    def reader(self) -> LocalSchemaReader | HttpSchemaReader:
+    def reader(self) -> LocalSchemaReader | HttpSchemaReader | StdinSchemaReader:
         if self._reader is None:
             self._reader = self._get_schema_reader()
         return self._reader
 
-    def _get_schema_reader(self) -> LocalSchemaReader | HttpSchemaReader:
+    def _get_schema_reader(
+        self,
+    ) -> LocalSchemaReader | HttpSchemaReader | StdinSchemaReader:
+        if self.schemafile == "-":
+            return StdinSchemaReader()
+
         if self.url_info is None or self.url_info.scheme in ("file", ""):
             return LocalSchemaReader(self.schemafile)
 
@@ -104,11 +117,14 @@ class SchemaLoader(SchemaLoaderBase):
         return self.reader.get_retrieval_uri()
 
     def get_schema(self) -> dict[str, t.Any]:
-        return self.reader.read_schema()
+        data = self.reader.read_schema()
+        if self.base_uri is not None:
+            data["$id"] = self.base_uri
+        return data
 
     def get_validator(
         self,
-        path: pathlib.Path,
+        path: pathlib.Path | str,
         instance_doc: dict[str, t.Any],
         format_opts: FormatOptions,
         fill_defaults: bool,
@@ -127,9 +143,19 @@ class SchemaLoader(SchemaLoaderBase):
             self._parsers, retrieval_uri, schema
         )
 
-        # get the correct validator class and check the schema under its metaschema
-        validator_cls = jsonschema.validators.validator_for(schema)
-        validator_cls.check_schema(schema)
+        if self.validator_class is None:
+            # get the correct validator class and check the schema under its metaschema
+            validator_cls = jsonschema.validators.validator_for(schema)
+            validator_cls.check_schema(schema)
+        else:
+            # for a user-provided validator class, don't check_schema
+            # on the grounds that it might *not* be valid but the user wants to use
+            # their custom validator anyway
+            #
+            # in fact, there's no real guarantee that a user-provided
+            # validator_class properly conforms to the jsonschema.Validator protocol
+            # we *hope* that it does, but we can't be fully sure
+            validator_cls = self.validator_class
 
         # extend the validator class with default-filling behavior if appropriate
         if fill_defaults:
@@ -145,21 +171,32 @@ class SchemaLoader(SchemaLoaderBase):
 
 
 class BuiltinSchemaLoader(SchemaLoader):
-    def __init__(self, schema_name: str) -> None:
+    def __init__(self, schema_name: str, base_uri: str | None = None) -> None:
         self.schema_name = schema_name
+        self.base_uri = base_uri
         self._parsers = ParserSet()
 
     def get_schema_retrieval_uri(self) -> str | None:
         return None
 
     def get_schema(self) -> dict[str, t.Any]:
-        return get_builtin_schema(self.schema_name)
+        data = get_builtin_schema(self.schema_name)
+        if self.base_uri is not None:
+            data["$id"] = self.base_uri
+        return data
 
 
 class MetaSchemaLoader(SchemaLoaderBase):
+    def __init__(self, base_uri: str | None = None) -> None:
+        if base_uri is not None:
+            raise NotImplementedError(
+                "'--base-uri' was used with '--metaschema'. "
+                "This combination is not supported."
+            )
+
     def get_validator(
         self,
-        path: pathlib.Path,
+        path: pathlib.Path | str,
         instance_doc: dict[str, t.Any],
         format_opts: FormatOptions,
         fill_defaults: bool,
